;; This is the lut_NameInput table of character codes, in the order the codes
;; appear on the "Select Name" screen, each code also being paired with a
;; string that shows what character it represents.
(local CHARACTER-MAP
  [0x8a "あ" 0x8b "い" 0x8c "う" 0x8d "え" 0x8e "お" 0x48 "が" 0x49 "ぎ" 0x4a "ぐ" 0x4b "げ" 0x4c "ご"
   0x8f "か" 0x90 "き" 0x91 "く" 0x92 "け" 0x93 "こ" 0x4d "ざ" 0x4e "じ" 0x4f "ず" 0x50 "ぜ" 0x51 "ぞ"
   0x94 "さ" 0x95 "し" 0x96 "す" 0x97 "せ" 0x98 "そ" 0x52 "だ" 0x53 "ぢ" 0x54 "づ" 0x55 "で" 0x56 "ど"
   0x99 "た" 0x9a "ち" 0x9b "つ" 0x9c "て" 0x9d "と" 0x57 "ば" 0x58 "び" 0x59 "ぶ" 0x5a "べ" 0x5b "ぼ"
   0x9e "な" 0x9f "に" 0xa0 "ぬ" 0xa1 "ね" 0xa2 "の" 0x70 "ぱ" 0x71 "ぴ" 0x72 "ぷ" 0x73 "ぺ" 0x74 "ぽ"
   0xa3 "は" 0xa4 "ひ" 0xa5 "ふ" 0xa6 "へ" 0xa7 "ほ" 0x7d "ゃ" 0x7e "ゅ" 0x7f "ょ" 0x7c "っ" 0xb9 "。"
   0xa8 "ま" 0xa9 "み" 0xaa "む" 0xab "め" 0xac "も" 0x80 "０" 0x81 "１" 0x82 "２" 0x83 "３" 0x84 "４"
   0xb0 "ら" 0xb1 "り" 0xb2 "る" 0xb3 "れ" 0xb4 "ろ" 0x85 "５" 0x86 "６" 0x87 "７" 0x88 "８" 0x89 "９"
   0xad "や" 0xae "ゆ" 0xaf "よ" 0xb5 "わ" 0xb6 "ん" 0xc2 "－" 0xc4 "！" 0xc5 "？" 0xc3  "‥" 0xff "　"])
(local NAME-ENTRY-WIDTH 10)
(local NAME-ENTRY-HEIGHT 9)

;; Convert the UTF-8 string s into a list of character codes. Returns nil plus
;; an error message if the string cannot be encoded.
(local encode-text
  (do
    ;; Build character→code table for fast lookup.
    (var enc {})
    (for [i 1 (length CHARACTER-MAP) 2]
      (let [n (. CHARACTER-MAP (+ i 0))
            c (. CHARACTER-MAP (+ i 1))]
        (assert (= 1 (assert (utf8.len c) c)) c)
        (assert (= nil (. enc c)) c)
        (tset enc c n)))
    ;; Throw in a few more characters that should also map to existing codes.
    (each [c n (pairs {"'" 0xbe
                       "−" 0xc2 "–" 0xc2})]
      (assert (= 1 (assert (utf8.len c) c)) c)
      (assert (= nil (. enc c)) c)
      (tset enc c n))
    (lambda [s]
      (case (pcall #(icollect [c (string.gmatch s utf8.charpattern)]
                      (or (. enc c) (error c 0))))
        (true x) x
        (_ c) (values nil (string.format "no encoding for %q" c))))))

{:VERSION :jp
 :CODE
 {:NewGamePartyGeneration 0x9c54
  :PtyGen_DrawScreen 0x9cf8
  :DoPartyGen_OnCharacter 0x9d15
  :DoNameInput 0x9d50
  :EnterTitleScreen 0xa16f
  :EnterMainMenu 0xadc6
  :MainMenuLoop_B_Pressed 0xae15
  :DoOverworld 0xc0cb
  :ScreenWipe_Finalize 0xd72c
  :CyclePalettes_Done 0xd966}
 :VAR
 {:ow_scroll_x 0x0027
  :ow_scroll_y 0x0028
  :sm_scroll_x 0x0029
  :sm_scroll_y 0x002a
  :mapflags 0x002d
  :move_speed 0x0034
  :move_ctr_x 0x0035
  :move_ctr_y 0x0036
  :cursor 0x0062
  :namecurs_x 0x0064
  :namecurs_y 0x0065
  :respondrate 0x00fa
  :ptygen [{:class 0x0300 :name 0x0302}
           {:class 0x0310 :name 0x0312}
           {:class 0x0320 :name 0x0322}
           {:class 0x0330 :name 0x0332}]}
 :CONST
 {
  ;; Character classes.
  :CLS {:FT  0 :TH  1 :BB  2 :RM  3 :WM  4 :BM  5
        :KN  6 :NJ  7 :MA  8 :RW  9 :WW 10 :BW 11}
  ;; Just the character codes from CHARACTER-MAP.
  :lut_NameInput
  (fcollect [i 1 (length CHARACTER-MAP) 2] (. CHARACTER-MAP i))}
 : NAME-ENTRY-WIDTH
 : NAME-ENTRY-HEIGHT
 : encode-text}
