;; Unload our own modules, to avoid old versions being cached in package.loaded
;; from a previous execution.
(each [_ modname (ipairs [:ff1 :ff1jp :game :savestate :ucs :util])]
  (tset package.loaded modname nil))
;; Here we load a version-specific ff1 module, then additionally store it as
;; package.loaded.ff1. Other modules will require it simply as :ff1.
(local ff1 (require :ff1jp)) (tset package.loaded :ff1 ff1)
(local game (require :game))
(local
  {: run
   : restore-savestate!}
  (require :savestate))

(local {: Up : Down} game)

;; Call f iteratively n times, starting with the value init.
(lambda times [init n f]
  (faccumulate [acc init _ 1 n] (f acc)))

(lambda main [savestate]
  (-> savestate
      ;; Do the title menu and party selection.
      (game.title-menu 0)
      (game.party-select {:class ff1.CONST.CLS.FT
                          :name [0x8b 0x90 0x91 0x96]}  ; いきくす
                         {:class ff1.CONST.CLS.TH
                          :name [0xaf 0xb2 0x8d 0x90]}  ; よるえき
                         {:class ff1.CONST.CLS.BB
                          :name [0x4c 0x51 0x50 0x55]}  ; ごぞぜで
                         {:class ff1.CONST.CLS.RM
                          :name [0x9a 0x4c 0x9f 0xaa]}) ; ちごにむ
      ;; Enter the overworld.
      (game.await-screen-wipe)

      ;; Enter Coneria Castle.
      (times 6 #(game.move-on-map $ Up))
      (game.await-screen-wipe-2)
      ;; Approach the stairs.
      (times 16 #(game.move-on-map $ Up))

      ;; Start the repeated stair traversals.
      (times 18 #(-> $
                     (game.move-on-map Up)      ; Step onto stairs.
                     (game.await-screen-wipe-2) ; Screen transition.
                     (game.move-on-map Down)))  ; Step off of stairs.
      ;; Open and close the menu.
      (game.open-menu)  (game.await-palette-cycle)
      (game.close-menu) (game.await-palette-cycle)
      ;; More stair traversals.
      (times 13 #(-> $
                     (game.move-on-map Up)      ; Step onto stairs.
                     (game.await-screen-wipe-2) ; Screen transition.
                     (game.move-on-map Down)))  ; Step off of stairs.

      ;; One last stair traversal.
      (game.move-on-map Up)
      (game.await-screen-wipe-2)
      ;; Open the menu.
      (game.open-menu)

      ;; Sync the emulator state to the final computed savestate, in order to
      ;; save a movie.
      (restore-savestate!)))

(assert (run main))
