(local heap (require :heap))
(import-macros {: lexico-lt?} :macros)

(lambda comp [a b]
  (lexico-lt? [a.cost a.serial] [b.cost b.serial]))

(lambda uniform-cost-search [start key finished? neighbors]
  ;; Attach an incrementing serial number to each priority queue entry, and
  ;; sort by both cost and serial number, in order to keep the heap stable with
  ;; respect to insertion order.
  (var serial 0)
  (lambda inc-serial [] (set serial (+ serial 1)) serial)

  (let [queue []
        traceback {}]
    (heap.insert queue comp
                 {:state start :prev nil :cost 0 :serial (inc-serial)})
    (var winner nil)
    (each [{: state : prev : cost} (heap.iter queue comp) &until winner]
      (let [k (key state)]
        (if (not (. traceback k))
          (do
            (tset traceback k prev)
            (if (finished? state) (set winner state))
            (each [(nstate edge-cost) (neighbors state)]
              (assert (>= edge-cost 0) edge-cost)
              (if (not (. traceback (key nstate)))
                (heap.insert queue comp
                             {:state nstate :prev state :cost (+ cost edge-cost) :serial (inc-serial)})))))))

    ;; Trace back the winning state to the start via the traceback pointers.
    (if winner
      (let [path []]
        (var state winner)
        (while state
          (table.insert path state)
          (set state (. traceback (key state))))
        ;; Reverse the list.
        (fcollect [i (length path) 1 -1] (. path i))))))

{: uniform-cost-search}
