;; Consolidate all tables provided as arguments into a new table, with values
;; in later tables taking precedence.
(lambda table-update [...]
  (accumulate [new {} _ other (ipairs [...])]
    (collect [k v (pairs other) &into new] (values k v))))

;; Return a shallow copy of t, with additional entries in ... appended to its
;; sequential keys.
(lambda table-append [t ...]
  (let [new (collect [k v (pairs t)] (values k v))]
    (icollect [_ v (ipairs [...]) &into new] v)))

;; An iterator like ipairs that endlessly repeats a sequential table.
(lambda irepeat [a]
  (lambda iter [a i]
    (let [i (+ (% i (length a)) 1)]
      (values i (. a i))))
  (assert (> (length a) 0))
  (values iter a (length a)))

{: table-update
 : table-append
 : irepeat}
